const fs = require("fs");
const path = require("path");
const AWS = require("aws-sdk");
const rimraf = require("rimraf");
const zipFolder = require("zip-folder");
const ncp = require("ncp");

const lambda = new AWS.Lambda();
var iam = new AWS.IAM();

const self = {
  P: function (fc) {
    return new Promise(function (resolve, reject) {
      try {
        fc(resolve);
      } catch (error) {
        reject(error);
      }
    });
  },
  exists: async function (filename) {
    return new Promise(function (resolve, rejcet) {
      try {
        fs.exists(path.resolve(filename), function (exists) {
          return resolve(exists);
        });
      } catch (e) {
        rejcet(e);
      }
    });
  },
  getIndexFile: function (fullpath) {
    if (
      fullpath.indexOf("/index.js") === -1 &&
      fullpath.indexOf("index.js") === -1
    ) {
      return `${fullpath}/index.js`;
    } else if (
      fullpath.indexOf("/index.js") === -1 &&
      fullpath.indexOf("index.js") !== -1
    ) {
      return `${fullpath}/index.js`;
    }
  },

  validatorParams: function (params) {
    let target, mode, layer_list;
    if (!params[0]) {
      throw new Error("파라미터가 비어있습니다.");
    }
    if (params[0][0] === "-" && params[0] !== "--layer" && params[0] !== "-l") {
      throw new Error(`wrong parameter. fix '${params[0]}' to '-layer'`);
    }
    if (params[0] === "-l" || params[0] === "--layer") {
      mode = "layer";
      target = params[1];
      layer_list = [];
    } else {
      mode = "lambda";
      target = params.shift();
      layer_list = params;
    }

    if (!self.exists(path.resolve(target))) {
      throw new Error(
        ["대상 디렉토리가 없습니다.", `target: ${target}`].join("\n")
      );
    }
    if (layer_list && layer_list.length) {
      for (let i = 0; i < layer_list[i]; i++) {
        if (!self.exists(path.resolve(layer_list[i]))) {
          throw new Error(
            ["대상 디렉토리가 없습니다.", `target: ${layer_list[i]}`].join("\n")
          );
        }
      }
    }
    return {
      mode,
      target,
      layer_list,
    };
  },
  createRole: async function (name) {
    const role = await self.getRole(`${name}-player`);
    if (role) {
      return role;
    }

    return new Promise((resolve, reject) => {
      iam.createRole(
        {
          AssumeRolePolicyDocument: `{
            "Version": "2012-10-17",
            "Statement": [
              {
                "Effect": "Allow",
                "Principal": {
                  "Service": "lambda.amazonaws.com"
                },
                "Action": "sts:AssumeRole"
              }
            ]
          }`,
          RoleName: `${name}-player`,
        },
        function (err, data) {
          if (err) {
            console.log(err, err.stack);
            reject(err);
          } else {
            iam.attachRolePolicy(
              {
                PolicyArn: "arn:aws:iam::aws:policy/service-role/AWSLambdaRole",
                RoleName: `${name}-player`,
              },
              function (err, _data) {
                if (err) {
                  console.log(err, err.stack);
                  resolve({
                    Role: {
                      Arn: data.Role.Arn,
                      AssumeRolePolicyDocument:
                        data.Role.AssumeRolePolicyDocument,
                      CreateDate: data.Role.CreateDate,
                      Path: data.Role.Path,
                      RoleId: data.Role.RoleId,
                      RoleName: data.Role.RoleName,
                    },
                  });
                } else {
                  resolve({
                    Role: {
                      Arn: data.Role.Arn,
                      AssumeRolePolicyDocument:
                        data.Role.AssumeRolePolicyDocument,
                      CreateDate: data.Role.CreateDate,
                      Path: data.Role.Path,
                      RoleId: data.Role.RoleId,
                      RoleName: data.Role.RoleName,
                    },
                  });
                }
              }
            );
          }
        }
      );
    });
  },
  getRole: function (name) {
    return new Promise(function (resolve, reject) {
      iam.getRole(
        {
          RoleName: name,
        },
        function (err, data) {
          if (err) {
            // console.log(err, err.stack);
            return resolve(null);
          }
          return resolve({
            Role: {
              Arn: data.Role.Arn,
              AssumeRolePolicyDocument: data.Role.AssumeRolePolicyDocument,
              CreateDate: data.Role.CreateDate,
              MaxSessionDuration: data.Role.MaxSessionDuration,
              Path: data.Role.Path,
              RoleId: data.Role.RoleId,
              RoleLastUsed: data.Role.RoleLastUsed,
              RoleName: data.Role.RoleName,
            },
          });
        }
      );
    });
  },
  getBufferZipDir: async function (target) {
    return new Promise(async (resolve, reject) => {
      const temp_dir = new Date().getTime() + "_temp";
      const temp_file = new Date().getTime() + ".zip";
      await fs.mkdirSync(`./${temp_dir}`);

      zipFolder(path.resolve(target), `./${temp_dir}/${temp_file}`, (err) => {
        if (err) {
          return reject(
            new Error(
              ["압축 실패했습니다.", `${JSON.stringify(err, null, 2)}`].join(
                "\n"
              )
            )
          );
        }
        resolve(fs.readFileSync(`./${temp_dir}/${temp_file}`));
        rimraf(`./${temp_dir}`, function () {});
      });
    });
  },
  copyDir: async (src, dst) => {
    return new Promise((resolve, reject) => {
      ncp(src, dst, (err) => {
        if (err) {
          return reject(
            new Error(
              [
                "디렉토리 복사 실패했습니다.",
                `${JSON.stringify(err, null, 2)}`,
              ].join("\n")
            )
          );
        }
        resolve(true);
      });
    });
  },

  makeLayerSoureZipToBuffer: async (target) => {
    const copy_target = target + "_copy";
    await self.copyDir(path.resolve(target), path.resolve(copy_target));

    const temp_dir = new Date().getTime() + "_target";
    await fs.renameSync(
      path.resolve(copy_target),
      path.resolve(`${copy_target}/../${temp_dir}`)
    );
    await fs.mkdirSync(path.resolve(`${copy_target}`));
    await fs.mkdirSync(path.resolve(`${copy_target}/nodejs`));
    await fs.renameSync(
      path.resolve(`${copy_target}/../${temp_dir}`),
      path.resolve(`${copy_target}/nodejs/node_modules`)
    );
    const data = await self.getBufferZipDir(copy_target);
    rimraf(path.resolve(`${copy_target}`), function () {});
    return data;
  },
  makeSoureZipToBuffer: async (target) => {
    return await self.getBufferZipDir(target);
  },
  createLayer: async (target, name) => {
    const data = await self.makeLayerSoureZipToBuffer(target);
    var params = {
      Content: {
        ZipFile: data,
      },
      LayerName: name,
      CompatibleRuntimes: ["nodejs10.x"],
    };
    return new Promise((resolve, reject) =>
      lambda.publishLayerVersion(params, function (err, data) {
        if (err) {
          console.error(err, err.stack);
          return reject(new Error("createFunction 실패"));
        }
        resolve(data);
      })
    );
  },
  createFunction: async (target, name, role_arn) => {
    const data = await self.makeSoureZipToBuffer(target);
    return new Promise((resolve, reject) =>
      lambda.createFunction(
        {
          Code: {
            ZipFile: data,
          },
          FunctionName: name,
          Handler: "index.handler",
          Role: role_arn,
          Runtime: "nodejs10.x",
          Publish: true,
        },
        function (err, data) {
          if (err) {
            console.error(err, err.stack);
            return reject(new Error("createFunction 실패"));
          }
          resolve(data);
        }
      )
    );
  },
};
module.exports = self;
